#define _WIN32_DCOM

#include <windows.h>
#include "../Commun/Fabriques--ServeurTempsQuiPasse.h"
#include "../Commun/IConfigTemps.h"
#include <olectl.h>    			// Pour les interfaces de point de connexion
#include <string>
#include <iostream>

#include "ClientDeRappel.h"
#include "../../StdOutilsCOM_Lib/StdOutilsClient.h"


int main (int argc, char *argv[])
{
   IKitConnexion *pIKC;
   pIKC = StdInitialisationClient (argc > 1? argv[1] : 0);
   if (!pIKC)
   {
      std::cerr << "ERREUR: DCOM non install" << std::endl;
      return -1;
   }

   // Cration du client de rappel. On veut un ITempsQuiPasse *, peu
   // importe le dtail de son implmentation. Avoir t plus gentil,
   // j'aurais mis ce qui suit dans une fonction...
   // La raison pour laquelle il y a un bloc ici est pour dlimiter
   // la porte de pRappel, qui en sert que trs localement. Voyons
   // cela comme une espce de mini fabrique...
   ITempsQuiPasse *pTempsQuiPasse = 0;
   {
      ClientDeRappel *pRappel = new ClientDeRappel ();
      pRappel->AddRef ();
      if (FAILED (pRappel->QueryInterface
                    (IID_ITempsQuiPasse, 
                     reinterpret_cast<void **> (&pTempsQuiPasse))))
      {
         std::cerr << "ERREUR interne (QueryInterface() du client de rappel)" << std::endl;
         StdFermetureClient (pIKC);
         return -2;
      }
      pRappel->Release ();
   }


   // Cration du serveur. On veut un IConnectionPointContainer* ...
   HRESULT hr;
   MULTI_QI mqi [1] = { &__uuidof(IConnectionPointContainer), 0, S_OK };
   hr = CoCreateInstanceEx (CLSID_FabriqueServeurTempsQuiPasse, 0,
                            pIKC->GetContexte (),
                            pIKC->GetInfoServeur (), 1, mqi);
   if (FAILED (hr))
   {
      std::cerr << "Erreur de fabrication du serveur. Code: "
                << hr
                << std::endl;
      StdFermetureClient (pIKC);
      return -3;
   }
   IConnectionPointContainer *piCPC = static_cast<IConnectionPointContainer *>(mqi[0].pItf);

   // Obtention du point de connexion. Une fois le conteneur obtenu, on
   // lui demande un pointeur vers le point de connexion pour notre interface...
   IConnectionPoint *pCP;
   if (SUCCEEDED (piCPC->FindConnectionPoint (IID_ITempsQuiPasse, &pCP)))
   {
      // Avis au serveur: rappelle-moi!
      DWORD dwCookie;
      if (SUCCEEDED (pCP->Advise (pTempsQuiPasse, &dwCookie)))
      {
         std::cout << "Pressez une touche, puis ENTER pour terminer..."
                   << std::endl;
         char c;
         std::cin >> c;
         // Avis au serveur: ne me rappelle plus!
         pCP->Unadvise (dwCookie);
      }
      pCP->Release ();
      piCPC->Release ();
   }
   pTempsQuiPasse->Release ();
   StdFermetureClient (pIKC);
   return 0;
}