#define _WIN32_DCOM

#include <windows.h>
#include "../Commun/Fabriques--ServeurTempsQuiPasse.h"
#include "../Commun/IConfigTemps.h"
#include <string>
#include <iostream>

#include "../../StdOutilsCOM_Lib/StdOutilsClient.h"


const char CHOIX_TICSPARUNITE = 'T',
           CHOIX_UNITESPARSAISON = 'U',
           CHOIX_QUITTER = 'Q';

// Affiche la configuration courante de l'horloge
void AfficherConfigCourante (IConfigTemps *);

// Affiche le menu de cet utilitaire
void AfficherMenu ();

// Retourne VRAI si le choix pass en paramtre est
// un choix valide, FAUX sinon
bool ChoixEstValide (const char);

// Lit et retourne un choix (non valid)
char LireChoix ();

// Traite le choix reu en paramtre (valid) 
// l'aide du pointeur de IConfigTemps (valide)
void TraiterChoix (IConfigTemps *, const char);

int main (int argc, char *argv[])
{
   IKitConnexion *pIKC;
   pIKC = StdInitialisationClient (argc > 1? argv[1] : 0);
   if (!pIKC)
   {
      std::cerr << "ERREUR: DCOM non install" << std::endl;
      return -1;
   }
   // On ne demande qu'une seule interface: IConfigTemps
   HRESULT hr;
   MULTI_QI mqi [1] = { &IID_IConfigTemps, 0, S_OK };
   hr = CoCreateInstanceEx (CLSID_FabriqueServeurTempsQuiPasse, 0,
                            pIKC->GetContexte (),
                            pIKC->GetInfoServeur (), 1, mqi);
   if (FAILED (hr))
   {
      std::cerr << "Erreur de fabrication du serveur. Code: "
                << hr
                << std::endl;
      StdFermetureClient (pIKC);
      return -3;
   }

   IConfigTemps *pConfigTemps = static_cast<IConfigTemps *>(mqi[0].pItf);

   // Utilisation de l'interface
   AfficherConfigCourante (pConfigTemps);
   // ...
   char Choix;
   do
   {
      AfficherMenu ();
      Choix = LireChoix ();
   }
   while (!ChoixEstValide (Choix));
   while (Choix != CHOIX_QUITTER)
   {
      TraiterChoix (pConfigTemps, Choix);
      AfficherConfigCourante (pConfigTemps);
      do
      {
         AfficherMenu ();
         Choix = LireChoix ();
      }
      while (!ChoixEstValide (Choix));
   }

   pConfigTemps->Release ();
   StdFermetureClient (pIKC);
   return 0;
}


// Affiche la configuration courante de l'horloge
void AfficherConfigCourante (IConfigTemps *pConfigTemps)
{
   long TicsParUnite,
        UnitesParSaison;

   pConfigTemps->GetUniteDeTemps(&TicsParUnite);
   pConfigTemps->GetUnitesParSaison(&UnitesParSaison);
   std::cout << std::endl << std::endl << std::endl
             << "Horloge -- Configuration courante" << std::endl
             << "Nombre de tics par unite de temps: "
             << TicsParUnite
             << std::endl
             << "Nombre d'unites de temps par saison: "
             << UnitesParSaison
             << std::endl;
}

// Affiche le menu de cet utilitaire
void AfficherMenu ()
{
   std::cout << "---------------------" << std::endl
             << "[" << CHOIX_TICSPARUNITE << "] Modifier les tics par unite de temps?" << std::endl
             << "[" << CHOIX_UNITESPARSAISON << "] Modifier les unites de temps par saison?" << std::endl
             << "[" << CHOIX_QUITTER << "] Quitter?" << std::endl
             << "---------------------" << std::endl;
}

// Retourne VRAI si le choix pass en paramtre est
// un choix valide, FAUX sinon
bool ChoixEstValide (const char Choix)
{
   return Choix == CHOIX_TICSPARUNITE ||
          Choix == CHOIX_UNITESPARSAISON ||
          Choix == CHOIX_QUITTER;
}

// Lit et retourne un choix (non valid)
char LireChoix ()
{
   char Choix;
   std::cin >> Choix;
   return toupper (Choix);
}

// Traite le choix reu en paramtre (valid) 
// l'aide du pointeur de IConfigTemps (valide)
void TraiterChoix (IConfigTemps *pConfigTemps, const char Choix)
{
   switch (Choix)
   {
      case CHOIX_TICSPARUNITE:
      {
         long NouvelleValeurTicsParUnite;
         std::cout << "Nombre de tics par unit de temps dsirs? ";
         std::cin >> NouvelleValeurTicsParUnite;
         while (NouvelleValeurTicsParUnite <= 0)
         {
            std::cout << "La valeur doit tre strictement positiive..."
                      << std::endl
                      << "Nombre de tics par unit de temps dsirs? ";
            std::cin >> NouvelleValeurTicsParUnite;
         }
         pConfigTemps->SetUniteDeTemps (NouvelleValeurTicsParUnite);
      }
      break;
      case CHOIX_UNITESPARSAISON:
      {
         long NouvelleValeurUnitesParSaison;
         std::cout << "Nombre d'units de temps par saison dsires? ";
         std::cin >> NouvelleValeurUnitesParSaison;
         while (NouvelleValeurUnitesParSaison <= 0)
         {
            std::cout << "La valeur doit tre strictement positiive..."
                      << std::endl
                      << "Nombre d'units de temps par saison dsires? ";
            std::cin >> NouvelleValeurUnitesParSaison;
         }
         pConfigTemps->SetUnitesParSaison (NouvelleValeurUnitesParSaison);
      }
      break;
   }
}
