#ifndef ENUMERATEURDECONNEXIONS_H
#define ENUMERATEURDECONNEXIONS_H

#include <vector>
#include <olectl.h>
#include "../../StdOutilsCOM_Lib/IUnknownImpl.h"
#include "../Commun/IConfigTemps.h"



// ----------------------------------------------

class EnumerateurDeConnexions : public virtual IUnknownImpl
{
public:
class Connexion
{
   CONNECTDATA m_StdConnectData;
public:
   unsigned long GetCookie () const
   {
      return m_StdConnectData.dwCookie;
   }
   IUnknown *GetInterface ()
   {
      m_StdConnectData.pUnk->AddRef ();
      return m_StdConnectData.pUnk;
   }
   Connexion (IUnknown *pUnk, const unsigned long Cookie)
   {
      m_StdConnectData.pUnk = pUnk;
      m_StdConnectData.pUnk->AddRef ();
      m_StdConnectData.dwCookie = Cookie;
   }
   const Connexion & operator = (const Connexion &p)
   {
      m_StdConnectData.dwCookie = p.GetCookie ();
      m_StdConnectData.pUnk = const_cast<Connexion &>(p).GetInterface ();
      return *this;
   }
   bool operator == (const Connexion &p) const
   {
      return GetCookie () == p.GetCookie ();
   }
   bool operator != (const Connexion &p) const
   {
      return GetCookie () != p.GetCookie ();
   }
   virtual ~Connexion ()
   {
      m_StdConnectData.pUnk->Release ();
   }
};
private:
   std::vector<Connexion *> m_PointsDeConnexion;
   std::vector<Connexion *>::iterator m_Itt;
   int GetCurPos () const
   {
      int Resultat = 0;
      std::vector<Connexion *>::const_iterator itt;
      for (itt = m_PointsDeConnexion.begin (); itt != m_Itt; itt++)
      {
         Resultat ++;
      }
      return Resultat;
   }
public:
   // Pour IUnknown
   unsigned long __stdcall AddRef () { return IUnknownImpl::AddRef (); }
   unsigned long __stdcall Release () { return IUnknownImpl::Release (); }
   HRESULT __stdcall QueryInterface (REFIID riid, void **ppv)
   {
      if (riid == IID_IUnknown)
      {
         *ppv = static_cast<IUnknown*> (static_cast<IUnknownImpl*> (this));
      }
      else
      {
         *ppv = 0;
         return E_NOINTERFACE;
      }
      AddRef ();
      return S_OK;
   }
   int GetNbConnexions () const
   {
      return m_PointsDeConnexion.size ();
   }
   void Ajouter (Connexion *pConnexion)
   {
      m_PointsDeConnexion.push_back (pConnexion);
   }
   std::vector<Connexion *>::const_iterator TrouverCookie(DWORD CookieProspectif) const
   {
      std::vector<Connexion *>::const_iterator itt;
      for (itt = m_PointsDeConnexion.begin (); itt != m_PointsDeConnexion.end () && (*itt)->GetCookie () != CookieProspectif; itt++);
      return itt;
   }
   std::vector<Connexion *>::iterator TrouverCookie(DWORD CookieProspectif)
   {
      std::vector<Connexion *>::iterator itt;
      for (itt = m_PointsDeConnexion.begin (); itt != m_PointsDeConnexion.end () && (*itt)->GetCookie () != CookieProspectif; itt++);
      return itt;
   }
   bool CookieExiste (DWORD CookieProspectif) const
   {
      return TrouverCookie (CookieProspectif) != m_PointsDeConnexion.end ();
   }
   bool Supprimer (DWORD Cookie)
   {
      std::vector<Connexion *>::iterator itt = TrouverCookie (Cookie);
      if (itt == m_PointsDeConnexion.end ())
         return false;
      Connexion *Temp = *itt;
      m_PointsDeConnexion.erase (itt);
      delete Temp;
      return true;
   }
   // constructeurs
   EnumerateurDeConnexions () : IUnknownImpl ()
   {
      m_Itt = m_PointsDeConnexion.begin ();
   }
   EnumerateurDeConnexions (const std::vector<Connexion *> &Pts) : IUnknownImpl ()
   {
      std::vector<Connexion *>::const_iterator itt;
      for (itt = Pts.begin (); itt != Pts.end (); itt++)
      {
         Ajouter (new Connexion (*(*itt)));
      }
      m_Itt = m_PointsDeConnexion.begin ();
   }
   EnumerateurDeConnexions (const EnumerateurDeConnexions &e) : IUnknownImpl ()
   {
      std::vector<Connexion *>::const_iterator itt;
      for (itt = e.m_PointsDeConnexion.begin (); itt != e.m_PointsDeConnexion.end (); itt++)
      {
         Ajouter (new Connexion (*(*itt)));
      }
      
      m_Itt = m_PointsDeConnexion.begin (); 
      for (int CptPos = e.GetCurPos (); --CptPos >= 0;)
      {
         m_Itt++;
      }
   }
   // destructeur
   ~EnumerateurDeConnexions ()
   {
      std::vector<Connexion *>::iterator itt;
      for (itt = m_PointsDeConnexion.begin ();
           itt != m_PointsDeConnexion.end ();
           itt++)
      {
         Connexion *Temp = *itt;
         itt++;
         delete Temp;
      }
   }
   CONNECTDATA *GenererLLRep () const;
};



#endif // !defined ENUMERATEURDECONNEXIONS_H