#ifndef ICONFIGTEMPS_H
#define ICONFIGTEMPS_H

#include "../Commun/IConfigTemps.h"
#include <iostream>

class IConfigTempsImpl : virtual public IConfigTemps
{
   static const long
      TICSPARUNITEDETEMPS_DEFAUT,
      UNITESDETEMPSPARSAISON_DEFAUT;
   // Paramtres dcrivant les intervalles. Son des attributs de
   // classe pour qu'ils soient partags par tous les serveurs,
   //  l'excution
   long m_TicsParUniteDeTemps,
        m_UnitesDeTempsParSaison;
   bool m_EtatVientDeChanger;
public:
   // accesseurs
   long GetTicsParUniteDeTemps () const
   {
      return m_TicsParUniteDeTemps;
   }
   long GetUnitesDeTempsParSaison () const
   {
      return m_UnitesDeTempsParSaison;
   }
private:
   // Compteurs d'units et de saison
   long m_NbUnites,
        m_NbSaisons;
public:
   // accesseurs
   long GetNbUnites () const
   {
      return m_NbUnites;
   }
   void ProchaineUnite ()
   {
      m_NbUnites ++;
      m_EtatVientDeChanger = true;
      m_TicDerniereUnite = GetPlusRecentTic ();
      if (abs (GetNbUnites () - GetUniteDerniereSaison ()) >= GetUnitesDeTempsParSaison ())
      {
         ProchaineSaison ();
      }
   }
   long GetNbSaisons () const
   {
      return m_NbSaisons;
   }
   void ProchaineSaison ()
   {
      m_NbSaisons ++;
      m_UniteDerniereSaison = GetNbUnites ();
   }
private:
   // Mmoire
   long m_TicDerniereUnite,
        m_UniteDerniereSaison,
        m_PlusRecentTic;
public:
   // accesseurs
   long GetTicDerniereUnite () const
   {
      return m_TicDerniereUnite;
   }
   long GetUniteDerniereSaison () const
   {
      return m_UniteDerniereSaison;
   }
   long GetPlusRecentTic () const
   {
      return m_PlusRecentTic;
   }

   // oprations lors du passage du temps
   void ProchainTic ()
   {
      m_PlusRecentTic ++;
      long TicsCourant = GetPlusRecentTic ();
      long TicsDernier = GetTicDerniereUnite ();
      long diff = abs (TicsCourant - TicsDernier);

      if (diff >= GetTicsParUniteDeTemps ())
      {
         ProchaineUnite ();
      }
   }

   bool ChangementEtat () const
   {
      return m_EtatVientDeChanger;
   }
   void ResetEtat ()
   {
      m_EtatVientDeChanger = false;
   }

   // Outils de validation
   static bool EstNbTicsParUniteDeTempsValide (const long NbTics)
   {
      return NbTics > 0;
   }
   static bool EstNbUnitesParSaisonValide (const long NbUnites)
   {
      return NbUnites > 0;
   }

   // Pour IConfigTemps
   HRESULT STDMETHODCALLTYPE GetUniteDeTemps(long __RPC_FAR *NbTics)
   {
      *NbTics = GetTicsParUniteDeTemps ();
      return S_OK;
   }
   HRESULT STDMETHODCALLTYPE SetUniteDeTemps(long NbTics)
   {
      if (!EstNbTicsParUniteDeTempsValide (NbTics))
         return E_INVALIDARG;
      m_TicsParUniteDeTemps = NbTics;
      return S_OK;
   }

   HRESULT STDMETHODCALLTYPE GetUnitesParSaison(long __RPC_FAR *NbUnites)
   {
      *NbUnites = GetUnitesDeTempsParSaison ();
      return S_OK;
   }
   HRESULT STDMETHODCALLTYPE SetUnitesParSaison(long NbUnites)
   {
      if (!EstNbUnitesParSaisonValide (NbUnites))
         return E_INVALIDARG;
      m_UnitesDeTempsParSaison = NbUnites;
      return S_OK;
   }

   // Mcanique de base
   IConfigTempsImpl () :
      m_EtatVientDeChanger (false),
      m_TicsParUniteDeTemps (TICSPARUNITEDETEMPS_DEFAUT),
      m_UnitesDeTempsParSaison (UNITESDETEMPSPARSAISON_DEFAUT)
   {
      m_PlusRecentTic = 0;
      m_TicDerniereUnite = 0;
      m_UniteDerniereSaison = 0;
      m_NbUnites = 0;
      m_NbSaisons = 0;
   }
   virtual ~IConfigTempsImpl ()
   {
   }

};

#endif // !defined ICONFIGTEMPS_H
