#include "IConnectionPointImpl.h"
#include "../../StdOutilsCOM_Lib/IUnknownImpl.h"
#include "../../StdOutils/SingletonGenerateurNombresAleatoires.h"
#include "EnumerateurDeConnexions.h"

#include <iostream>
#include <algorithm>


// Le point de connexion
IConnectionPointImpl::IConnectionPointImpl () : IConnectionPoint ()
{
}

IConnectionPointImpl::~IConnectionPointImpl ()
{
}


// Pour gestion interne
unsigned long IConnectionPointImpl::GenererCookieUnique () const
{
   int CookieProspectif = SingletonGenerateurNombresAleatoires::GetInstance ()->GetNombre ();
   while (CookieExiste (CookieProspectif))
   {
      CookieProspectif = SingletonGenerateurNombresAleatoires::GetInstance ()->GetNombre ();
   }
   return CookieProspectif;
}

bool IConnectionPointImpl::CookieExiste (const unsigned long Cookie) const
{
   return m_Connexions.CookieExiste (Cookie);
}

// Pour IConnectionPoint

// -- OK
HRESULT STDMETHODCALLTYPE IConnectionPointImpl::GetConnectionInterface(IID *pIID)
{
   if (!pIID)
      return E_POINTER;
   *pIID = GetInterfaceIID ();
   return S_OK;
}

// -- OK
HRESULT STDMETHODCALLTYPE IConnectionPointImpl::GetConnectionPointContainer(IConnectionPointContainer **ppCPC)
{
   IConnectionPointContainer *Parent = GetParent ();

   if (!Parent)
      return E_NOTIMPL;
   return Parent->QueryInterface (IID_IConnectionPointContainer, reinterpret_cast<void **>(ppCPC));
}

// -- OK
HRESULT STDMETHODCALLTYPE IConnectionPointImpl::Advise
   (IUnknown *pUnkSink, DWORD *pdwCookie)
{

   if (!pUnkSink || !pdwCookie)
      return E_POINTER;
   ITempsQuiPasse *pITQP;
   HRESULT hr = pUnkSink->QueryInterface (IID_ITempsQuiPasse,
                                          reinterpret_cast<void **>(&pITQP));
   if (FAILED (hr))
      return CONNECT_E_CANNOTCONNECT;
   pUnkSink->Release ();

   *pdwCookie = GenererCookieUnique ();

   EnumerateurDeConnexions::Connexion *pConnexion = new EnumerateurDeConnexions::Connexion (pITQP, *pdwCookie);
   if (!pConnexion)
      return E_OUTOFMEMORY;
   m_Connexions.Ajouter (pConnexion);
   return S_OK;
}

// -- OK
HRESULT STDMETHODCALLTYPE IConnectionPointImpl::Unadvise(DWORD dwCookie)
{
   if (!m_Connexions.Supprimer (dwCookie))
      return E_INVALIDARG;
   return S_OK;
}

void Relacher (CONNECTDATA &p)
{
   p.pUnk -> Release ();
}


HRESULT STDMETHODCALLTYPE IConnectionPointImpl::EnumConnections( IEnumConnections **ppEnum)
{
   class LLEnumConnection : public virtual IUnknownImpl,
                            public virtual IEnumConnections
   {
      CONNECTDATA *m_cd;
      int m_Taille;
      unsigned int m_Cur; // lment courant
   public:
      unsigned long __stdcall AddRef () { return IUnknownImpl::AddRef (); }
      unsigned long __stdcall Release () { return IUnknownImpl::Release (); }
      HRESULT __stdcall QueryInterface (REFIID riid, void **ppv)
      {
         if (riid == IID_IUnknown)
         {
            *ppv = static_cast<IUnknown *> (static_cast<IUnknownImpl *> (this));
         }
         else if (riid == IID_IEnumConnections)
         {
            *ppv = static_cast<IEnumConnections *> (this);
         }
         else
         {
            *ppv = 0;
            return E_NOINTERFACE;
         }
         AddRef ();
         return S_OK;
      }
      LLEnumConnection (CONNECTDATA *cd, const int Taille) : m_Taille (Taille)
      {
         m_cd = new CONNECTDATA [m_Taille];
         for (int i = 0; i < m_Taille; i++)
         {
            m_cd[i]= cd[i];
            m_cd[i].pUnk->AddRef ();
         }
         m_Cur = 0;
      }
      LLEnumConnection (const LLEnumConnection &llec) : m_Taille (llec.m_Taille)
      {
         m_cd = new CONNECTDATA [m_Taille];
         for (int i = 0; i < m_Taille; i++)
         {
            m_cd[i]= llec.m_cd[i];
            m_cd[i].pUnk->AddRef ();
         }
         m_Cur = llec.m_Cur;
      }
      ~LLEnumConnection ()
      {
         std::for_each (&m_cd[0], &m_cd[m_Taille], Relacher);
         delete [] m_cd;
      }
      // Pour IEnumConnections
	   HRESULT __stdcall Next(ULONG cConnexions, CONNECTDATA* rgpcd, ULONG* pcFetched)
      {
	      if(!pcFetched && cConnexions != 1)
		      return E_INVALIDARG;
	      if(pcFetched)
		      *pcFetched = 0;
	      if(!rgpcd || m_Cur >= m_Taille)
        	   return S_FALSE;
	      unsigned int cReturn = 0;
	      while(m_Cur < m_Taille && cConnexions > 0)
	      {
            *rgpcd++ = m_cd[m_Cur];
            m_cd[m_Cur].pUnk->AddRef();
            m_Cur++;
            cReturn++;
            cConnexions--;
	      } 
	      if(pcFetched != NULL)
        	   *pcFetched = cReturn;
	      return S_OK;
      }

	   HRESULT __stdcall Skip(ULONG cConnexions)
      {
         if(m_Cur + cConnexions >= m_Taille)
            return S_FALSE;
         m_Cur += cConnexions;
         return S_OK;
      }
	   HRESULT __stdcall Reset()
      {
         m_Cur = 0;
         return S_OK;
      }
	   HRESULT __stdcall Clone(IEnumConnections** ppEnum)
      {
	      if(!ppEnum)
		      return E_POINTER;

          // Cloner
         *ppEnum = new LLEnumConnection (*this);
         if(!*ppEnum)
            return E_OUTOFMEMORY;

         (*ppEnum)->AddRef();
         return S_OK;
      }
   };

   CONNECTDATA *TempCD = m_Connexions.GenererLLRep ();
   if (!TempCD)
      return E_OUTOFMEMORY;
   *ppEnum = new LLEnumConnection (TempCD, m_Connexions.GetNbConnexions ());
   delete [] TempCD;
   if (!*ppEnum)
      return E_OUTOFMEMORY;
   HRESULT hr;
   hr= (*ppEnum)->QueryInterface (IID_IEnumConnections,
                                  reinterpret_cast <void **> (ppEnum));
   (*ppEnum)->Release ();
   return hr;
}

